@@ -58,35 +58,6 @@ diagview<eT>::diagview(Mat<eT>& in_m, const u32 in_row_offset, const u32 in_col_
 
 
 
-//! set a diagonal of our matrix using data from a foreign object
-template<typename eT>
-template<typename T1>
-inline
-void
-diagview<eT>::operator= (const Base<eT,T1>& o)
-  {
-  arma_extra_debug_sigprint();
-  arma_check( (m_ptr == 0), "diagview::operator=(): matrix is read only");
-  
-  const unwrap<T1> tmp(o.get_ref());
-  const Mat<eT>& x = tmp.M;
-  
-  diagview& t = *this;
-  
-  arma_debug_check( !x.is_vec(), "diagview::operator=(): need a vector");
-  arma_debug_check( (t.n_elem != x.n_elem), "diagview::operator=(): diagonal and given vector have incompatible lengths");
-  
-  Mat<eT>& t_m = *(t.m_ptr);
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    t_m.at(i+row_offset, i+col_offset) = x.mem[i];
-    }
-  
-  }
-
-
-
 //! set a diagonal of our matrix using a diagonal from a foreign matrix
 template<typename eT>
 inline
@@ -94,7 +65,6 @@ void
 diagview<eT>::operator= (const diagview<eT>& x)
   {
   arma_extra_debug_sigprint();
-  arma_check( (m_ptr == 0), "diagview::operator=(): matrix is read only");
   
   diagview<eT>& t = *this;
   
@@ -103,16 +73,197 @@ diagview<eT>::operator= (const diagview<eT>& x)
         Mat<eT>& t_m = *(t.m_ptr);
   const Mat<eT>& x_m = x.m;
   
-  for(u32 i=0; i<n_elem; ++i)
+  const u32 t_n_elem     = t.n_elem;
+  const u32 t_row_offset = t.row_offset;
+  const u32 t_col_offset = t.col_offset;
+  
+  const u32 x_row_offset = x.row_offset;
+  const u32 x_col_offset = x.col_offset;
+  
+  for(u32 i=0; i<t_n_elem; ++i)
     {
-    t_m.at(i+t.row_offset, i+t.col_offset) = x_m.at(i+x.row_offset, i+x.col_offset);
+    t_m.at(i + t_row_offset, i + t_col_offset) = x_m.at(i + x_row_offset, i + x_col_offset);
+    }
+  }
+
+
+
+//! set a diagonal of our matrix using data from a foreign object
+template<typename eT>
+template<typename T1>
+inline
+void
+diagview<eT>::operator= (const Base<eT,T1>& o)
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap<T1> tmp(o.get_ref());
+  const Mat<eT>& x = tmp.M;
+  
+  diagview<eT>& t = *this;
+  
+  arma_debug_check
+    (
+    ( (t.n_elem != x.n_elem) || (x.is_vec() == false) ),
+    "diagview::operator=(): given object has incompatible size"
+    );
+  
+  Mat<eT>& t_m = *(t.m_ptr);
+  
+  const u32 t_n_elem     = t.n_elem;
+  const u32 t_row_offset = t.row_offset;
+  const u32 t_col_offset = t.col_offset;
+  
+  const eT* x_mem = x.memptr();
+  
+  for(u32 i=0; i<t_n_elem; ++i)
+    {
+    t_m.at( i + t_row_offset,  i + t_col_offset) = x_mem[i];
+    }
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+diagview<eT>::operator+=(const Base<eT,T1>& o)
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap<T1> tmp(o.get_ref());
+  const Mat<eT>& x = tmp.M;
+  
+  diagview<eT>& t = *this;
+  
+  arma_debug_check
+    (
+    ( (t.n_elem != x.n_elem) || (x.is_vec() == false) ),
+    "diagview::operator=(): given object has incompatible size"
+    );
+  
+  Mat<eT>& t_m = *(t.m_ptr);
+  
+  const u32 t_n_elem     = t.n_elem;
+  const u32 t_row_offset = t.row_offset;
+  const u32 t_col_offset = t.col_offset;
+  
+  const eT* x_mem = x.memptr();
+  
+  for(u32 i=0; i<t_n_elem; ++i)
+    {
+    t_m.at( i + t_row_offset,  i + t_col_offset) += x_mem[i];
+    }
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+diagview<eT>::operator-=(const Base<eT,T1>& o)
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap<T1> tmp(o.get_ref());
+  const Mat<eT>& x = tmp.M;
+  
+  diagview<eT>& t = *this;
+  
+  arma_debug_check
+    (
+    ( (t.n_elem != x.n_elem) || (x.is_vec() == false) ),
+    "diagview::operator=(): given object has incompatible size"
+    );
+  
+  Mat<eT>& t_m = *(t.m_ptr);
+  
+  const u32 t_n_elem     = t.n_elem;
+  const u32 t_row_offset = t.row_offset;
+  const u32 t_col_offset = t.col_offset;
+  
+  const eT* x_mem = x.memptr();
+  
+  for(u32 i=0; i<t_n_elem; ++i)
+    {
+    t_m.at( i + t_row_offset,  i + t_col_offset) -= x_mem[i];
+    }
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+diagview<eT>::operator%=(const Base<eT,T1>& o)
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap<T1> tmp(o.get_ref());
+  const Mat<eT>& x = tmp.M;
+  
+  diagview<eT>& t = *this;
+  
+  arma_debug_check
+    (
+    ( (t.n_elem != x.n_elem) || (x.is_vec() == false) ),
+    "diagview::operator=(): given object has incompatible size"
+    );
+  
+  Mat<eT>& t_m = *(t.m_ptr);
+  
+  const u32 t_n_elem     = t.n_elem;
+  const u32 t_row_offset = t.row_offset;
+  const u32 t_col_offset = t.col_offset;
+  
+  const eT* x_mem = x.memptr();
+  
+  for(u32 i=0; i<t_n_elem; ++i)
+    {
+    t_m.at( i + t_row_offset,  i + t_col_offset) *= x_mem[i];
+    }
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+diagview<eT>::operator/=(const Base<eT,T1>& o)
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap<T1> tmp(o.get_ref());
+  const Mat<eT>& x = tmp.M;
+  
+  diagview<eT>& t = *this;
+  
+  arma_debug_check
+    (
+    ( (t.n_elem != x.n_elem) || (x.is_vec() == false) ),
+    "diagview::operator=(): given object has incompatible size"
+    );
+  
+  Mat<eT>& t_m = *(t.m_ptr);
+  
+  const u32 t_n_elem     = t.n_elem;
+  const u32 t_row_offset = t.row_offset;
+  const u32 t_col_offset = t.col_offset;
+  
+  const eT* x_mem = x.memptr();
+  
+  for(u32 i=0; i<t_n_elem; ++i)
+    {
+    t_m.at( i + t_row_offset,  i + t_col_offset) /= x_mem[i];
     }
-    
   }
 
 
 
-//! \brief
 //! extract a diagonal and store it as a column vector
 template<typename eT>
 inline
@@ -299,7 +450,6 @@ arma_inline
 eT&
 diagview<eT>::operator()(const u32 i)
   {
-  arma_check( (m_ptr == 0), "diagview::operator(): matrix is read only");
   arma_debug_check( (i >= n_elem), "diagview::operator(): out of bounds" );
   
   return (*m_ptr).at(i+row_offset, i+col_offset);
@@ -324,7 +474,6 @@ arma_inline
 eT&
 diagview<eT>::operator()(const u32 row, const u32 col)
   {
-  arma_check( (m_ptr == 0), "diagview::operator(): matrix is read only");
   arma_debug_check( ((row >= n_elem) || (col > 0)), "diagview::operator(): out of bounds" );
   
   return (*m_ptr).at(row+row_offset, row+col_offset);
@@ -350,7 +499,6 @@ void
 diagview<eT>::fill(const eT val)
   {
   arma_extra_debug_sigprint();
-  arma_check( (m_ptr == 0), "diagview::fill(): matrix is read only");
   
   Mat<eT>& x = (*m_ptr);
   
@@ -368,7 +516,6 @@ void
 diagview<eT>::zeros()
   {
   arma_extra_debug_sigprint();
-  arma_check( (m_ptr == 0), "diagview::zeros(): matrix is read only");
   
   Mat<eT>& x = (*m_ptr);
   
@@ -386,7 +533,6 @@ void
 diagview<eT>::ones()
   {
   arma_extra_debug_sigprint();
-  arma_check( (m_ptr == 0), "diagview::ones(): matrix is read only");
   
   Mat<eT>& x = (*m_ptr);
   
