@@ -22,7 +22,7 @@
 template<typename T1>
 inline
 const Op<T1, op_chol>
-chol(const Base<typename T1::elem_type,T1>& X)
+chol(const Base<typename T1::elem_type,T1>& X, const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0)
   {
   arma_extra_debug_sigprint();
   
@@ -34,7 +34,7 @@ chol(const Base<typename T1::elem_type,T1>& X)
 template<typename T1>
 inline
 bool
-chol(Mat<typename T1::elem_type>& out, const Base<typename T1::elem_type,T1>& X)
+chol(Mat<typename T1::elem_type>& out, const Base<typename T1::elem_type,T1>& X, const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0)
   {
   arma_extra_debug_sigprint();
   
