@@ -24,7 +24,12 @@ template<typename T1>
 inline
 arma_warn_unused
 u32
-rank(const Base<typename T1::elem_type,T1>& X, typename T1::pod_type tol = 0.0)
+rank
+  (
+  const Base<typename T1::elem_type,T1>& X,
+  typename T1::pod_type tol = 0.0,
+  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0
+  )
   {
   arma_extra_debug_sigprint();
   
