@@ -31,9 +31,9 @@ class mtGlue : public Base<out_eT, mtGlue<out_eT, T1, T2, glue_type> >
   arma_inline  mtGlue(const T1& in_A, const T2& in_B, const u32 in_aux_u32);
   arma_inline ~mtGlue();
   
-  const T1& A;       //!< first operand
-  const T2& B;       //!< second operand
-  const u32 aux_u32; //!< storage of auxiliary data, u32 format
+  arma_aligned const T1& A;       //!< first operand
+  arma_aligned const T2& B;       //!< second operand
+  arma_aligned const u32 aux_u32; //!< storage of auxiliary data, u32 format
   };
 
 
