@@ -73,7 +73,7 @@ op_dot::direct_dot(const u32 n_elem, const eT* const A, const eT* const B)
       }
     #elif defined(ARMA_USE_BLAS)
       {
-      const int n = n_elem;
+      const blas_int n = n_elem;
       return blas::dot_(&n, A, B);
       }
     #else
@@ -199,27 +199,42 @@ op_dot::apply_proxy(const Base<typename T1::elem_type,T1>& X, const Base<typenam
   {
   arma_extra_debug_sigprint();
   
-  typedef typename T1::elem_type eT;
+  typedef typename T1::elem_type      eT;
+  typedef typename Proxy<T1>::ea_type ea_type1;
+  typedef typename Proxy<T2>::ea_type ea_type2;
   
   const Proxy<T1> A(X.get_ref());
   const Proxy<T2> B(Y.get_ref());
   
-  arma_debug_check( (A.n_elem != B.n_elem), "dot(): objects must have the same number of elements" );
+  arma_debug_check( (A.get_n_elem() != B.get_n_elem()), "dot(): objects must have the same number of elements" );
   
-  const u32 n_elem = A.n_elem;
-  eT val = eT(0);
+  const u32      N  = A.get_n_elem();
+        ea_type1 PA = A.get_ea();
+        ea_type2 PB = B.get_ea();
   
-  for(u32 i=0; i<n_elem; ++i)
+  eT val1 = eT(0);
+  eT val2 = eT(0);
+  
+  u32 i,j;
+  
+  for(i=0, j=1; j<N; i+=2, j+=2)
     {
-    val += A[i] * B[i];
+    val1 += PA[i] * PB[i];
+    val2 += PA[j] * PB[j];
     }
   
-  return val;
+  if(i < N)
+    {
+    val1 += PA[i] * PB[i];
+    }
+  
+  return val1 + val2;
   }
 
 
 
 //
+// op_norm_dot
 
 
 
@@ -231,76 +246,18 @@ op_norm_dot::apply(const Base<typename T1::elem_type,T1>& X, const Base<typename
   {
   arma_extra_debug_sigprint();
   
-  if( (is_Mat<T1>::value == true) && (is_Mat<T2>::value == true) )
-    {
-    return op_norm_dot::apply_unwrap(X,Y);
-    }
-  else
-    {
-    return op_norm_dot::apply_proxy(X,Y);
-    }
-  }
-
-
-
-template<typename T1, typename T2>
-arma_hot
-inline
-typename T1::elem_type
-op_norm_dot::apply_unwrap(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
-  {
-  arma_extra_debug_sigprint();
-  
-  typedef typename T1::elem_type eT;
-  
-  const unwrap<T1> tmp1(X.get_ref());
-  const unwrap<T2> tmp2(Y.get_ref());
-  
-  const Mat<eT>& A = tmp1.M;
-  const Mat<eT>& B = tmp2.M;
-
-  arma_debug_check( (A.n_elem != B.n_elem), "norm_dot(): objects must have the same number of elements" );
-  
-  const eT* A_mem = A.memptr();
-  const eT* B_mem = B.memptr();
-  
-  const u32 N = A.n_elem;
-  
-  eT acc1 = eT(0);
-  eT acc2 = eT(0);
-  eT acc3 = eT(0);
-  
-  for(u32 i=0; i<N; ++i)
-    {
-    const eT tmpA = A_mem[i];
-    const eT tmpB = B_mem[i];
-    
-    acc1 += tmpA * tmpA;
-    acc2 += tmpB * tmpB;
-    acc3 += tmpA * tmpB;
-    }
-    
-  return acc3 / ( std::sqrt(acc1 * acc2) );
-  }
-
-
-
-template<typename T1, typename T2>
-arma_hot
-inline
-typename T1::elem_type
-op_norm_dot::apply_proxy(const Base<typename T1::elem_type,T1>& X, const Base<typename T1::elem_type,T2>& Y)
-  {
-  arma_extra_debug_sigprint();
-  
-  typedef typename T1::elem_type eT;
+  typedef typename T1::elem_type      eT;
+  typedef typename Proxy<T1>::ea_type ea_type1;
+  typedef typename Proxy<T2>::ea_type ea_type2;
   
   const Proxy<T1> A(X.get_ref());
   const Proxy<T2> B(Y.get_ref());
-
-  arma_debug_check( (A.n_elem != B.n_elem), "norm_dot(): objects must have the same number of elements" );
   
-  const u32 N = A.n_elem;
+  arma_debug_check( (A.get_n_elem() != B.get_n_elem()), "norm_dot(): objects must have the same number of elements" );
+  
+  const u32      N  = A.get_n_elem();
+        ea_type1 PA = A.get_ea();
+        ea_type2 PB = B.get_ea();
   
   eT acc1 = eT(0);
   eT acc2 = eT(0);
@@ -308,8 +265,8 @@ op_norm_dot::apply_proxy(const Base<typename T1::elem_type,T1>& X, const Base<ty
   
   for(u32 i=0; i<N; ++i)
     {
-    const eT tmpA = A[i];
-    const eT tmpB = B[i];
+    const eT tmpA = PA[i];
+    const eT tmpB = PB[i];
     
     acc1 += tmpA * tmpA;
     acc2 += tmpB * tmpB;
