@@ -20,6 +20,7 @@
 
 //! Find the minimum value in an array
 template<typename eT>
+arma_pure
 inline 
 eT
 op_min::direct_min(const eT* const X, const u32 n_elem)
@@ -28,13 +29,33 @@ op_min::direct_min(const eT* const X, const u32 n_elem)
   
   eT min_val = X[0];
   
-  for(u32 i=1; i<n_elem; ++i)
+  u32 i,j;
+  
+  for(i=1, j=2; j<n_elem; i+=2, j+=2)
     {
-    const eT tmp_val = X[i];
+    const eT X_i = X[i];
     
-    if(tmp_val < min_val)
+    if(X_i < min_val)
       {
-      min_val = tmp_val;
+      min_val = X_i;
+      }
+    
+    const eT X_j = X[j];
+    
+    if(X_j < min_val)
+      {
+      min_val = X_j;
+      }
+    }
+  
+  
+  if(i < n_elem)
+    {
+    const eT X_i = X[i];
+    
+    if(X_i < min_val)
+      {
+      min_val = X_i;
       }
     }
   
@@ -51,9 +72,10 @@ op_min::direct_min(const subview<eT>& X)
   {
   arma_extra_debug_sigprint();
   
-  eT min_val = X[0];
+  const u32 X_n_elem = X.n_elem;
+        eT  min_val  = X[0];
   
-  for(u32 i=1; i<X.n_elem; ++i)
+  for(u32 i=1; i<X_n_elem; ++i)
     {
     eT tmp_val = X[i];
     
@@ -76,9 +98,10 @@ op_min::direct_min(const diagview<eT>& X)
   {
   arma_extra_debug_sigprint();
   
-  eT min_val = X[0];
+  const u32 X_n_elem = X.n_elem;
+        eT  min_val  = X[0];
   
-  for(u32 i=1; i<X.n_elem; ++i)
+  for(u32 i=1; i<X_n_elem; ++i)
     {
     eT tmp_val = X[i];
     
@@ -112,16 +135,18 @@ inline void op_min::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_min>&
   const u32 dim = in.aux_u32_a;
   arma_debug_check( (dim > 1), "min(): incorrect usage. dim must be 0 or 1");
   
+  const u32 X_n_rows = X.n_rows;
+  const u32 X_n_cols = X.n_cols;
   
   if(dim == 0)  // column-wise min
     {
     arma_extra_debug_print("op_min::apply(), dim = 0");
     
-    out.set_size(1, X.n_cols);
+    out.set_size(1, X_n_cols);
     
-    for(u32 col=0; col<X.n_cols; ++col)
+    for(u32 col=0; col<X_n_cols; ++col)
       {
-      out[col] = op_min::direct_min( X.colptr(col), X.n_rows );
+      out[col] = op_min::direct_min( X.colptr(col), X_n_rows );
       }
     }
   else
@@ -129,13 +154,13 @@ inline void op_min::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_min>&
     {
     arma_extra_debug_print("op_min::apply(), dim = 1");
     
-    out.set_size(X.n_rows, 1);
+    out.set_size(X_n_rows, 1);
     
-    for(u32 row=0; row<X.n_rows; ++row)
+    for(u32 row=0; row<X_n_rows; ++row)
       {
       eT min_val = X.at(row,0);
       
-      for(u32 col=1; col<X.n_cols; ++col)
+      for(u32 col=1; col<X_n_cols; ++col)
         {
         const eT tmp_val = X.at(row,col);
         
@@ -190,10 +215,11 @@ op_min::direct_min(const subview< std::complex<T> >& X)
   {
   arma_extra_debug_sigprint();
   
-  u32 index   = 0;
-  T   min_val = std::abs(X[index]);
+  const u32 X_n_elem = X.n_elem;
+        u32 index    = 0;
+        T   min_val  = std::abs(X[index]);
   
-  for(u32 i=1; i<X.n_elem; ++i)
+  for(u32 i=1; i<X_n_elem; ++i)
     {
     const T tmp_val = std::abs(X[i]);
     
@@ -217,10 +243,11 @@ op_min::direct_min(const diagview< std::complex<T> >& X)
   {
   arma_extra_debug_sigprint();
   
-  u32 index   = 0;
-  T   min_val = std::abs(X[index]);
+  const u32 X_n_elem = X.n_elem;
+        u32 index    = 0;
+        T   min_val  = std::abs(X[index]);
   
-  for(u32 i=1; i<X.n_elem; ++i)
+  for(u32 i=1; i<X_n_elem; ++i)
     {
     const T tmp_val = std::abs(X[i]);
     
@@ -253,16 +280,18 @@ inline void op_min::apply(Mat< std::complex<T> >& out, const Op<T1,op_min>& in)
   const u32 dim = in.aux_u32_a;
   arma_debug_check( (dim > 1), "min(): incorrect usage. dim must be 0 or 1");
   
+  const u32 X_n_rows = X.n_rows;
+  const u32 X_n_cols = X.n_cols;
   
   if(dim == 0)  // column-wise min
     {
     arma_extra_debug_print("op_min::apply(), dim = 0");
     
-    out.set_size(1, X.n_cols);
+    out.set_size(1, X_n_cols);
     
-    for(u32 col=0; col<X.n_cols; ++col)
+    for(u32 col=0; col<X_n_cols; ++col)
       {
-      out[col] = op_min::direct_min( X.colptr(col), X.n_rows );
+      out[col] = op_min::direct_min( X.colptr(col), X_n_rows );
       }
     }
   else
@@ -270,9 +299,9 @@ inline void op_min::apply(Mat< std::complex<T> >& out, const Op<T1,op_min>& in)
     {
     arma_extra_debug_print("op_min::apply(), dim = 1");
     
-    out.set_size(X.n_rows, 1);
+    out.set_size(X_n_rows, 1);
     
-    for(u32 row=0; row<X.n_rows; ++row)
+    for(u32 row=0; row<X_n_rows; ++row)
       {
       u32 index   = 0;
       T   min_val = std::abs(X.at(row,index));
