@@ -26,6 +26,8 @@ class op_real
   template<typename T1>
   inline static void apply( Mat<typename T1::pod_type>& out, const mtOp<typename T1::pod_type, T1, op_real>& X);
   
+  template<typename T1>
+  inline static void apply( Cube<typename T1::pod_type>& out, const mtOpCube<typename T1::pod_type, T1, op_real>& X);
   };
 
 
@@ -37,6 +39,8 @@ class op_imag
   template<typename T1>
   inline static void apply( Mat<typename T1::pod_type>& out, const mtOp<typename T1::pod_type, T1, op_imag>& X);
   
+  template<typename T1>
+  inline static void apply( Cube<typename T1::pod_type>& out, const mtOpCube<typename T1::pod_type, T1, op_imag>& X);
   };
 
 
@@ -48,6 +52,8 @@ class op_abs
   template<typename T1>
   inline static void apply( Mat<typename T1::pod_type>& out, const mtOp<typename T1::pod_type, T1, op_abs>& X);
   
+  template<typename T1>
+  inline static void apply( Cube<typename T1::pod_type>& out, const mtOpCube<typename T1::pod_type, T1, op_abs>& X);
   };
 
 
