@@ -81,42 +81,10 @@ operator-
 
 
 
-//! non-complex Base - complex scalar (experimental)
-template<typename T1>
-arma_inline
-Mat<typename std::complex<typename T1::pod_type> >
-operator-
-  (
-  const Base<typename T1::pod_type, T1>&     X,
-  const std::complex<typename T1::pod_type>& k
-  )
-  {
-  arma_extra_debug_sigprint();
-  
-  typedef typename std::complex<typename T1::pod_type> eT;
-  typedef typename T1::pod_type                         T;
-  
-  const Proxy<T1> A(X.get_ref());
-  
-  Mat<eT> out(A.n_rows, A.n_cols);
-  
-  const u32 n_elem  = A.n_elem;
-        eT* out_mem = out.memptr();
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    out_mem[i] = A[i] - k;
-    }
-  
-  return out;
-  }
-
-
-
 //! complex scalar - non-complex Base (experimental)
 template<typename T1>
 arma_inline
-Mat<typename std::complex<typename T1::pod_type> >
+const mtOp<typename std::complex<typename T1::pod_type>, T1, op_cx_scalar_minus_pre>
 operator-
   (
   const std::complex<typename T1::pod_type>& k,
@@ -125,22 +93,24 @@ operator-
   {
   arma_extra_debug_sigprint();
   
-  typedef typename std::complex<typename T1::pod_type> eT;
-  typedef typename T1::pod_type                         T;
+  return mtOp<typename std::complex<typename T1::pod_type>, T1, op_cx_scalar_minus_pre>('j', X.get_ref(), k);
+  }
+
+
+
+//! non-complex Base - complex scalar (experimental)
+template<typename T1>
+arma_inline
+const mtOp<typename std::complex<typename T1::pod_type>, T1, op_cx_scalar_minus_post>
+operator-
+  (
+  const Base<typename T1::pod_type, T1>&     X,
+  const std::complex<typename T1::pod_type>& k
+  )
+  {
+  arma_extra_debug_sigprint();
   
-  const Proxy<T1> A(X.get_ref());
-  
-  Mat<eT> out(A.n_rows, A.n_cols);
-  
-  const u32 n_elem  = A.n_elem;
-        eT* out_mem = out.memptr();
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    out_mem[i] = k - A[i];
-    }
-  
-  return out;
+  return mtOp<typename std::complex<typename T1::pod_type>, T1, op_cx_scalar_minus_post>('j', X.get_ref(), k);
   }
 
 
@@ -164,8 +134,8 @@ operator-
 
 //! subtraction of Base objects with different element types
 template<typename T1, typename T2>
-arma_inline
-Mat<typename promote_type<typename T1::elem_type, typename T2::elem_type>::result>
+inline
+const mtGlue<typename promote_type<typename T1::elem_type, typename T2::elem_type>::result, T1, T2, glue_mixed_minus>
 operator-
   (
   const Base< typename force_different_type<typename T1::elem_type, typename T2::elem_type>::T1_result, T1>& X,
@@ -181,22 +151,7 @@ operator-
   
   promote_type<eT1,eT2>::check();
   
-  const Proxy<T1> A(X.get_ref());
-  const Proxy<T2> B(Y.get_ref());
-  
-  arma_debug_assert_same_size(A, B, "matrix subtraction");
-  
-  Mat<out_eT> out(A.n_rows, A.n_cols);
-
-        out_eT* out_mem = out.memptr();
-  const u32     n_elem  = out.n_elem;
-  
-  for(u32 i=0; i<n_elem; ++i)
-    {
-    out_mem[i] = upgrade_val<eT1,eT2>::apply(A[i]) - upgrade_val<eT1,eT2>::apply(B[i]);
-    }
-  
-  return out;
+  return mtGlue<out_eT, T1, T2, glue_mixed_minus>( X.get_ref(), Y.get_ref() );
   }
 
 
