@@ -74,6 +74,19 @@ template<> struct arma_signed_integral_only<long>   { typedef long   result; };
 
 
 
+template<typename T> struct arma_signed_only { };
+
+template<> struct arma_signed_only<char>   { typedef char   result; };
+template<> struct arma_signed_only<short>  { typedef short  result; };
+template<> struct arma_signed_only<int>    { typedef int    result; };
+template<> struct arma_signed_only<long>   { typedef long   result; };
+template<> struct arma_signed_only<float>  { typedef float  result; };
+template<> struct arma_signed_only<double> { typedef double result; };
+
+template<typename T> struct arma_signed_only< std::complex<T> > { typedef std::complex<T> result; };
+
+
+
 template<typename T> struct arma_float_only { };
 
 template<> struct arma_float_only<float>  { typedef float  result; };
@@ -100,6 +113,15 @@ template<typename T> struct arma_not_cx< std::complex<T> > { };
 
 
 
+template<typename T> struct arma_blas_type_only { };
+
+template<> struct arma_blas_type_only< float                > { typedef float                result; };
+template<> struct arma_blas_type_only< double               > { typedef double               result; };
+template<> struct arma_blas_type_only< std::complex<float>  > { typedef std::complex<float>  result; };
+template<> struct arma_blas_type_only< std::complex<double> > { typedef std::complex<double> result; };
+
+
+
 template<typename T> struct arma_op_rel_only { };
 
 template<> struct arma_op_rel_only< op_rel_lt_pre    > { typedef int result; };
@@ -141,4 +163,16 @@ template<> struct arma_glue_rel_only< glue_rel_noteq > { typedef int result; };
 
 
 
+template<typename T> struct arma_Mat_Col_Row_only { };
+
+template<typename eT> struct arma_Mat_Col_Row_only< Mat<eT> > { typedef Mat<eT> result; };
+template<typename eT> struct arma_Mat_Col_Row_only< Col<eT> > { typedef Col<eT> result; };
+template<typename eT> struct arma_Mat_Col_Row_only< Row<eT> > { typedef Row<eT> result; };
+
+
+
+template<typename  T> struct arma_Cube_only             { };
+template<typename eT> struct arma_Cube_only< Cube<eT> > { typedef Cube<eT> result; };
+
+
 //! @}
